// $Id: ValidationRule.cs 10 2008-02-29 15:58:02Z mcartoixa $

/* ***************************************************************************
 *                                                                           *
 * Copyright  2007 NourY Solutions                                          *
 *                                                                           *
 * This file is part of DataRules.                                           *
 *                                                                           *
 * DataRules is free software; you can redistribute it and/or modify it      *
 * under the terms of the GNU Lesser General Public License as published     *
 * by the Free Software Foundation; either version 3 of the License, or      *
 * (at your option) any later version.                                       *
 *                                                                           *
 * DataRules is distributed in the hope that it will be useful, but          *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU          *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with this program.  If not, see                             *
 * <http://www.gnu.org/licenses/lgpl.html>.                                  *
 *                                                                           *
 *************************************************************************** */

/*! \file
 * \brief Definition of \c Salamanca.DataRules.EnterpriseLibrary.ValidationRule.
 *
 * \author $Author: mcartoixa $
 * \date $Date: 2008-02-29 16:58:02 +0100 (ven., 29 févr. 2008) $
 * \version $Revision: 10 $
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Validation;
using Microsoft.Practices.EnterpriseLibrary.Validation.Integration;

namespace Salamanca.DataRules.EnterpriseLibrary
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Business rule based on a <see cref="Validator" /> instance.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    internal class ValidationRule<T>:
        GenericRule<T>,
        IValidationIntegrationProxy
    {

        /// <summary>Initializes a new instance of the <see cref="ValidationRule{T}" /> class.</summary>
        /// <param name="propertyName">Name of the property.</param>
        public ValidationRule(string propertyName):
            this(propertyName, ValidationSpecificationSource.Both, string.Empty)
        {
        }

        /// <summary>Initializes a new instance of the <see cref="ValidationRule{T}" /> class.</summary>
        /// <param name="propertyName">Name of the property.</param>
        /// <param name="validationSource">The validation source.</param>
        public ValidationRule(string propertyName, ValidationSpecificationSource validationSource):
            this(propertyName, validationSource, string.Empty)
        {
        }

        /// <summary>Initializes a new instance of the <see cref="ValidationRule{T}" /> class.</summary>
        /// <param name="propertyName">Name of the property.</param>
        /// <param name="validationSource">The validation source.</param>
        /// <param name="ruleSet">The rule set.</param>
        public ValidationRule(string propertyName, ValidationSpecificationSource validationSource, string ruleSet):
            base(propertyName, string.Empty, RuleValidationMode.PropertySpecific)
        {
            _ValidationSource=validationSource;
            _RuleSet=ruleSet;
        }

        /// <summary>Validates the rule on the specified target.</summary>
        /// <param name="target">The instance against which the rule is validated.</param>
        /// <returns><c>true</c> if the instance is valid for the rule, <c>false</c> if not.</returns>
        protected override bool DoValidate(object target)
        {
            _Target=target;
            _Description=string.Empty;

            Validator validator=null;
            if (string.IsNullOrEmpty(PropertyName))
                validator=ValidationFactory.CreateValidator<T>();
            else
                validator=new ValidationIntegrationHelper(this).GetValidator();

            if (validator!=null)
            {
                ValidationResults results=validator.Validate(target);

                _Description=GetMessage(results);
                return results.IsValid;
            }

            return true;
        }

        private static string GetMessage(ValidationResults results)
        {
            StringBuilder ret=new StringBuilder();

            foreach (ValidationResult vr in results)
                AppendMessage(ret, vr, string.Empty);

            return ret.ToString();
        }

        private static void AppendMessage(StringBuilder sb, ValidationResult result, string indent)
        {
            sb.Append(indent);
            sb.AppendLine(result.Message);

            string ind=null;
            foreach (ValidationResult nvr in result.NestedValidationResults)
            {
                if (ind==null)
                    ind=new string('\t', indent.Length+1);
                AppendMessage(sb, nvr, ind);
            }
        }

        /// <summary>The description of the rule.</summary>
        public override string Description
        {
            get
            {
                if (string.IsNullOrEmpty(_Description))
                    return base.Description;

                return _Description;
            }
        }

        #region IValidationIntegrationProxy

        internal protected MemberValueAccessBuilder MemberValueAccessBuilder
        {
            get
            {
                if (_ValueAccessBuilder==null)
                    _ValueAccessBuilder=new PropertyValueAccessBuilder();
                return _ValueAccessBuilder;
            }
        }

        MemberValueAccessBuilder IValidationIntegrationProxy.GetMemberValueAccessBuilder()
        {
            return MemberValueAccessBuilder;
        }

        object IValidationIntegrationProxy.GetRawValue()
        {
            return _Target
                .GetType()
                .GetProperty(PropertyName, BindingFlags.Public | BindingFlags.Instance | BindingFlags.FlattenHierarchy)
                .GetValue(_Target, null);
                
        }

        void IValidationIntegrationProxy.PerformCustomValueConversion(ValueConvertEventArgs e)
        {
        }

        bool IValidationIntegrationProxy.ProvidesCustomValueConversion
        {
            get
            {
                return false;
            }
        }

        string IValidationIntegrationProxy.Ruleset
        {
            get
            {
                return _RuleSet;
            }
        }

        ValidationSpecificationSource IValidationIntegrationProxy.SpecificationSource
        {
            get
            {
                return _ValidationSource;
            }
        }

        string IValidationIntegrationProxy.ValidatedPropertyName
        {
            get
            {
                return PropertyName;
            }
        }

        Type IValidationIntegrationProxy.ValidatedType
        {
            get
            {
                return typeof(T);
            }
        }

        #endregion

        private MemberValueAccessBuilder _ValueAccessBuilder;
        private string _RuleSet;
        private ValidationSpecificationSource _ValidationSource;
        private object _Target;
        private string _Description;

    }
}
