// $Id: DataMapper.cs 10 2008-02-29 15:58:02Z mcartoixa $
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*! \file
 * \brief Generic web services oriented implementation of Data Mappers.
 *
 * \author $Author: mcartoixa $
 * \date $Date: 2008-02-29 16:58:02 +0100 (ven., 29 févr. 2008) $
 * \version $Revision: 10 $
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.Design.Serialization;
using System.Diagnostics;
using System.Reflection;
using System.Web.Services.Protocols;
using Salamanca.DataAccess.Collections;
using Salamanca.DataRules;

namespace Salamanca.DataAccess.Web
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Provides data for web services oriented <include file="../../doc/Global.doc.xml" path="doc/template[@name='DataMapper']/*" />s events.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public class DataMapperEventArgs<TDomainEntity>:
        EventArgs
        where TDomainEntity:
            IDomainEntity
    {

        /// <summary>Constructor.</summary>
        public DataMapperEventArgs(TDomainEntity domainModel)
        {
            DomainEntity=domainModel;
        }

        /// <summary>The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> involved.</summary>
        public TDomainEntity DomainEntity;
    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Ancestor of a web services oriented <include file="../../doc/Global.doc.xml" path="doc/template[@name='DataMapper']/*" /></summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public abstract class DataMapper<TDomainEntity, TPrimaryKey, TDataTransferObject>:
        Salamanca.DataAccess.DataMapperBase<TDomainEntity, TPrimaryKey, TDataTransferObject>
        where TDomainEntity:
            DomainEntity<TPrimaryKey, TDataTransferObject>
        where TPrimaryKey:
            IPrimaryKey,
            new()
        where TDataTransferObject:
            class, // SOAP serialization does not accept structs...
            IDataTransferObject,
            new()
    {

        /// <summary>Constructor.</summary>
        /// <param name="protocol"></param>
        /// <param name="instanceCreator">Member used to create new instances for the type. May be <c>null</c>, in which case the default constructor for the type will be used.</param>
        /// <param name="cacheManager">A cache manager. Must not be <c>null</c>.</param>
        protected DataMapper(SoapHttpClientProtocol protocol, DomainEntityCreator<TDomainEntity> instanceCreator, ICacheManager cacheManager):
            base(instanceCreator, cacheManager)
        {
            _Protocol=protocol;
        }


        /// <summary>Finds the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> associated to the specified key.</summary>
        public override TDomainEntity Find(TPrimaryKey key, DataMapperCollection dataMappers)
        {
            return Load<IDomainEntityDataTransferObject>(
                GetFinder(key, dataMappers),
                dataMappers
            );
        }


        /// <summary>Refreshes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance from the database.</summary>
        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be refreshed.</param>
        protected override void Refresh(TDomainEntity domainModel)
        {
            IDomainEntity dm=(IDomainEntity)domainModel;
            IDomainEntityDataTransferObject result=GetFinder((TPrimaryKey)dm.Id, dm.DataMappers)();
            SetData(domainModel, result);
        }

        /// <summary>Returns a delegate that finds the data associated to the spcified parameters.</summary>
        protected abstract FindSingleDelegate<IDomainEntityDataTransferObject> GetFinder(TPrimaryKey key, DataMapperCollection dataMappers);

        /// <summary>Returns the list of <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instances
        /// returned by the specified web services <paramref name="finder">delegates</paramref>.</summary>
        protected DomainEntityKeyedCollection<TDomainEntity> Load<TDomainEntityDataTransferObject>(FindMultipleDelegate<TDomainEntityDataTransferObject> finder, DataMapperCollection dataMappers)
            where TDomainEntityDataTransferObject:
                IDomainEntityDataTransferObject
        {
            TDomainEntityDataTransferObject[] result=finder();
            List<TDomainEntity> ret=new List<TDomainEntity>();

            foreach (TDomainEntityDataTransferObject d in result)
            {
                TDomainEntity m=GetDomainEntity(d, dataMappers);
                ret.Add(m);
            }

            return new DomainEntityKeyedCollection<TDomainEntity>(ret);
        }

        /// <summary>Returns the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance
        /// returned by the specified web services <paramref name="finder">delegate</paramref>.</summary>
        protected TDomainEntity Load<TDomainEntityDataTransferObject>(FindSingleDelegate<TDomainEntityDataTransferObject> finder, DataMapperCollection dataMappers)
            where TDomainEntityDataTransferObject:
                IDomainEntityDataTransferObject
        {
            IDomainEntityDataTransferObject result=finder();
            return GetDomainEntity(result, dataMappers);
        }

        /// <summary>Raises the <see cref="Refreshed" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnRefreshed(TDomainEntity domainModel)
        {
            if (Refreshed!=null)
                Refreshed(this, new DataMapperEventArgs<TDomainEntity>(domainModel));

            domainModel.TriggerRefreshedEvent(new EventArgs());
        }

        /// <summary>Raises the <see cref="Refreshing" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnRefreshing(TDomainEntity domainModel)
        {
            if (Refreshing!=null)
                Refreshing(this, new DataMapperEventArgs<TDomainEntity>(domainModel));

            domainModel.TriggerRefreshingEvent(new EventArgs());
        }



        /// <summary>Deletes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
//        /// <param name="data">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be deleted.</param>
        protected abstract IDomainEntityDataTransferObject Delete(IDomainEntityDataTransferObject data, bool force);

        /// <summary>Saves the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
//        /// <param name="data">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be saved.</param>
        protected abstract IDomainEntityDataTransferObject Save(IDomainEntityDataTransferObject data, bool force);

        /// <summary>Deletes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        //        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be deleted.</param>
        protected override void Delete(TDomainEntity domainModel, IPersistenceContext context)
        {
            Debug.Assert(context!=null);
            if (context==null)
                throw new ArgumentNullException("context");
            Debug.Assert(!((IDomainEntity)domainModel).IsNew);
            if (((IDomainEntity)domainModel).IsNew)
                throw new InvalidOperationException(Resources.SR.CannotDeleteNewDomainEntity);

            domainModel.TriggerDeletingEvent(new PersistenceEventArgs(context));

            if (context.ForceAction || !((IDomainEntity)domainModel).IsNew)
            {
                DoDeleting(domainModel, context);

                SetData(domainModel, (DomainEntityDataTransferObject<TDataTransferObject>)Delete(GetData((TDomainEntity)domainModel, false), context.ForceAction));

                DoDeleted(domainModel, context);
            }

            domainModel.TriggerDeletedEvent(new PersistenceEventArgs(context));
        }



        /// <summary>Saves the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        //        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be saved.</param>
        protected override void Save(TDomainEntity domainModel, IPersistenceContext context)
        {
            Debug.Assert(context!=null);
            if (context==null)
                throw new ArgumentNullException("context");

            domainModel.TriggerSavingEvent(new PersistenceEventArgs(context));

            if (context.ForceAction || (domainModel.IsModified && (((IDomainEntity)domainModel).IsNew || ((IDomainEntity)domainModel).IsLoaded)))
            {
                if (!domainModel.Validate())
                    throw new BrokenRulesException(((IValidatable)domainModel).GetBrokenRules(), domainModel);

                bool isNew=((IDomainEntity)domainModel).IsNew;
                if (isNew)
                    DoInserting(domainModel, context);
                else
                    DoUpdating(domainModel, context);

                SetData(domainModel, (DomainEntityDataTransferObject<TDataTransferObject>)Save(GetData(domainModel, false), context.ForceAction));

                if (isNew)
                    DoInserted(domainModel, context);
                else
                    DoUpdated(domainModel, context);
            }

            domainModel.TriggerSavedEvent(new PersistenceEventArgs(context));
        }



        private void DoDeleted(TDomainEntity domainModel, IPersistenceContext context)
        {
            domainModel.SetNew(true);
            domainModel.SetLoaded(false);

            OnDeleted(domainModel);

            CacheManager.Remove((TPrimaryKey)((IDomainEntity)domainModel).Id);
        }

        private void DoInserted(TDomainEntity domainModel, IPersistenceContext context)
        {
            domainModel.SetModified(false);
            domainModel.SetNew(false);
            domainModel.SetLoaded(true);

            OnInserted(domainModel);

            CacheManager.Add((TPrimaryKey)((IDomainEntity)domainModel).Id, (TDomainEntity)domainModel);
        }

        private void DoUpdated(TDomainEntity domainModel, IPersistenceContext context)
        {
            domainModel.SetModified(false);

            OnUpdated(domainModel);
        }

        private void DoDeleting(TDomainEntity domainModel, IPersistenceContext context)
        {
            OnDeleting(domainModel);
        }

        private void DoInserting(TDomainEntity domainModel, IPersistenceContext context)
        {
            OnInserting(domainModel);
        }

        private void DoUpdating(TDomainEntity domainModel, IPersistenceContext context)
        {
            OnUpdating(domainModel);
        }



        /// <summary>Raises the <see cref="Deleted" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnDeleted(TDomainEntity domainModel)
        {
            if (Deleted!=null)
                Deleted(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Raises the <see cref="Inserted" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnInserted(TDomainEntity domainModel)
        {
            if (Inserted!=null)
                Inserted(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Raises the <see cref="Updated" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnUpdated(TDomainEntity domainModel)
        {
            if (Updated!=null)
                Updated(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Raises the <see cref="Deleting" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnDeleting(TDomainEntity domainModel)
        {
            if (Deleting!=null)
                Deleting(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Raises the <see cref="Inserting" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnInserting(TDomainEntity domainModel)
        {
            if (Inserting!=null)
                Inserting(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Raises the <see cref="Updating" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnUpdating(TDomainEntity domainModel)
        {
            if (Updating!=null)
                Updating(this, new DataMapperEventArgs<TDomainEntity>(domainModel));
        }



        /// <summary>Gets the proxy client class.</summary>
        protected SoapHttpClientProtocol Protocol
        {
            get
            {
                return _Protocol;
            }
        }



        /// <summary>Delegate that returns the data of one <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        protected delegate TDomainEntityDataTransferObject FindSingleDelegate<TDomainEntityDataTransferObject>()
            where TDomainEntityDataTransferObject:
                IDomainEntityDataTransferObject;
        /// <summary>Delegate that returns the data of several <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instances.</summary>
        protected delegate TDomainEntityDataTransferObject[] FindMultipleDelegate<TDomainEntityDataTransferObject>()
            where TDomainEntityDataTransferObject:
                IDomainEntityDataTransferObject;

        private SoapHttpClientProtocol _Protocol;

        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> has been <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Refresh">refreshed</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Refreshed;
        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> is about to be <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Refresh">refreshed</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Refreshing;
        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> has been <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Delete()">deleted</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Deleted;
        /// <summary>Occurs when a <see cref="IDomainEntity.IsNew">new</see> <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> has been <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Save()">saved</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Inserted;
        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> has been <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Save()">saved</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Updated;
        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> is about to be <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Delete()">deleted</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Deleting;
        /// <summary>Occurs when a <see cref="IDomainEntity.IsNew">new</see> <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> is about to be <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Save()">saved</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Inserting;
        /// <summary>Occurs when a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> is about to be <see cref="DomainEntity{TPrimaryKey,TDataTransferObject}.Save()">saved</see>.</summary>
        public event EventHandler<DataMapperEventArgs<TDomainEntity>> Updating;

    }
}
