// $Id: KeyedCollection.cs 10 2008-02-29 15:58:02Z mcartoixa $
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*! \file
 * \brief Definition of Domain Entity keyed collections.
 *
 * \author $Author: mcartoixa $
 * \date $Date: 2008-02-29 16:58:02 +0100 (ven., 29 févr. 2008) $
 * \version $Revision: 10 $
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;

namespace Salamanca.DataAccess.Collections
{


    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Arguments of a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> collection saving event.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public sealed class DomainEntityCollectionSavingEventArgs<TDomainEntity>:
        PersistenceEventArgs
        where TDomainEntity:
            IDomainEntity
    {

        /// <summary>Constructor.</summary>
        public DomainEntityCollectionSavingEventArgs(IList<TDomainEntity> addedItems, IList<TDomainEntity> removedItems, IPersistenceContext persistenceContext):
            base(persistenceContext)
        {
            _AddedItems=addedItems;
            _RemovedItems=removedItems;
        }

        /// <summary>Items added to the collection since its last save.</summary>
        public IList<TDomainEntity> AddedItems
        {
            get
            {
                return _AddedItems;
            }
        }

        /// <summary>Items removed from the collection since its last save.</summary>
        public IList<TDomainEntity> RemovedItems
        {
            get
            {
                return _RemovedItems;
            }
        }

        /// <summary>Items added to the collection since its last save.</summary>
        private IList<TDomainEntity> _AddedItems;
        /// <summary>Items removed from the collection since its last save.</summary>
        private IList<TDomainEntity> _RemovedItems;

    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Implements a  <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> keyed collection.</summary>
    /// <remarks>
    /// <para>This collection is designed to handle new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instances and their temporary keys.
    /// An event handler is automatically registered to the <see cref="IPersistent.Saved" /> event, that triggers appropriate action when a definitive primary key
    /// is assigned to the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</para>
    /// </remarks>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public class DomainEntityKeyedCollection<TDomainEntity>:
        KeyedCollection<IPrimaryKey, TDomainEntity>,
        IPersistent
        where TDomainEntity:
            IDomainEntity
    {

        /// <summary>Constructor.</summary>
        public DomainEntityKeyedCollection():
            base()
        {
        }

        /// <summary>Constructor.</summary>
        public DomainEntityKeyedCollection(IEnumerable<TDomainEntity> collection):
            this()
        {
            foreach (TDomainEntity dm in collection)
                Add(dm);

            Reset();
        }

        /// <summary>Saves all the elements contained in the collection.</summary>
        public void Save(IPersistenceContext context)
        {
            Debug.Assert(context!=null);
            if (context==null)
                throw new ArgumentNullException("context");

            if (_SaveLock==null || !_SaveLock.Locked)
                using (_SaveLock=new PersistenceLock())
                {
                    OnSaving(
                        new DomainEntityCollectionSavingEventArgs<TDomainEntity>(
                            new ReadOnlyCollection<TDomainEntity>(_AddedItems.ToArray()), // ToArray() is the short way to make a copy
                            new ReadOnlyCollection<TDomainEntity>(_RemovedItems.ToArray()), // ToArray() is the short way to make a copy
                            context
                        )
                    );

                    foreach (TDomainEntity dm in _RemovedItems)
                        if (!dm.IsNew)
                            dm.Save(context);

                    // The keyed collection will be modified, so we have to make a copy first to prevent an exception in the following foreach statement
                    List<TDomainEntity> tbs=new List<TDomainEntity>(Items);
                    foreach (TDomainEntity dm in tbs)
                        dm.Save(context);

                    Reset();

                    OnSaved(new PersistenceEventArgs(context));
                }
        }

        /// <summary>Deletes all the elements contained in the collection.</summary>
        public void Delete(IPersistenceContext context)
        {
            Debug.Assert(context!=null);
            if (context==null)
                throw new ArgumentNullException("context");

            if (_DeleteLock==null || !_DeleteLock.Locked)
                using (_DeleteLock=new PersistenceLock())
                {
                    OnDeleting(new PersistenceEventArgs(context));

                    // The keyed collection will be modified, so we have to make a copy first to prevent an exception in the following foreach statement
                    List<TDomainEntity> tbd=new List<TDomainEntity>(Items);
                    foreach (TDomainEntity dm in tbd)
                        if (!dm.IsNew)
                            dm.Delete(context);

                    Reset();

                    OnDeleted(new PersistenceEventArgs(context));
                }
        }

        /// <summary>Removes all elements from the KeyedCollection.</summary>
        protected override void ClearItems()
        {
            List<TDomainEntity> items=new List<TDomainEntity>(Items);

            _RemovedItems.AddRange(Items);
            foreach (TDomainEntity dm in _AddedItems)
                _RemovedItems.Remove(dm);
            _AddedItems.Clear();

            base.ClearItems();

            if (CollectionChanged!=null)
                CollectionChanged(this, new CollectionChangeEventArgs(CollectionChangeAction.Refresh, items));
        }

        /// <summary>When implemented in a derived class, extracts the key from the specified element.</summary>
        protected override IPrimaryKey GetKeyForItem(TDomainEntity item)
        {
            return item.Id;
        }

        /// <summary>Inserts an element into the <see cref="DomainEntityKeyedCollection{TDomainEntity}" /> at the specified index.</summary>
        protected override void InsertItem(int index, TDomainEntity item)
        {
            RegisterEvents(item);

            base.InsertItem(index, item);

            if (CollectionChanged!=null)
                CollectionChanged(this, new CollectionChangeEventArgs(CollectionChangeAction.Add, item));
        }

        /// <summary>Removes the element at the specified index of the <see cref="DomainEntityKeyedCollection{TDomainEntity}" />.</summary>
        protected override void RemoveItem(int index)
        {
            TDomainEntity item=this[index];

            UnregisterEvents(item);

            base.RemoveItem(index);

            if (CollectionChanged!=null)
                CollectionChanged(this, new CollectionChangeEventArgs(CollectionChangeAction.Remove, item));
        }

        /// <summary>Replaces the item at the specified index with the specified item.</summary>
        protected override void SetItem(int index, TDomainEntity item)
        {
            TDomainEntity oldItem=this[index];

            UnregisterEvents(oldItem);
            RegisterEvents(item);

            base.SetItem(index, item);

            if (CollectionChanged!=null)
            {
                CollectionChanged(this, new CollectionChangeEventArgs(CollectionChangeAction.Remove, oldItem));
                CollectionChanged(this, new CollectionChangeEventArgs(CollectionChangeAction.Add, item));
            }
        }

        /// <summary>Raises the <see cref="Deleting" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnDeleting(PersistenceEventArgs e)
        {
            if (Deleting!=null)
                Deleting(this, e);
        }

        /// <summary>Raises the <see cref="Deleted" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnDeleted(PersistenceEventArgs e)
        {
            if (Deleted!=null)
                Deleted(this, e);
        }

        /// <summary>Raises the <see cref="Saving" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnSaving(DomainEntityCollectionSavingEventArgs<TDomainEntity> e)
        {
            if (_Saving!=null)
                _Saving(this, e);
        }

        /// <summary>Raises the <see cref="Saved" /> event.</summary>
        /// <remarks>
        ///   <include file="../../doc/Global.doc.xml" path="doc/template[@name='OnEvent']/*" />
        /// </remarks>
        protected virtual void OnSaved(PersistenceEventArgs e)
        {
            if (Saved!=null)
                Saved(this, e);
        }

        /// <summary>Registers new events to the specified item.</summary>
        private void RegisterEvents(TDomainEntity item)
        {
            if (!_RemovedItems.Remove(item))
                _AddedItems.Add(item);

            item.Deleted+=ItemDeleted;
            if (item.IsNew)
            {
                item.Saving+=ItemSaving;
                item.Saved+=ItemSaved;
            }
        }

        /// <summary>Unregisters the events from the specified item.</summary>
        private void UnregisterEvents(TDomainEntity item)
        {
            if (!_AddedItems.Remove(item))
                _RemovedItems.Add(item);

            item.Deleted-=ItemDeleted;
            item.Saving-=ItemSaving;
            item.Saved-=ItemSaved;
        }

        /// <summary>Action performed when an item of the collection is deleted.</summary>
        private void ItemDeleted(object sender, PersistenceEventArgs e)
        {
            IDomainEntity dm=(IDomainEntity)sender;
            if (dm!=null)
                dm.Deleted-=ItemDeleted;

            Remove((TDomainEntity)sender);
        }

        /// <summary>Action performed when an item of the collection is about to be saved.</summary>
        private void ItemSaving(object sender, PersistenceEventArgs e)
        {
            IDomainEntity dm=(IDomainEntity)sender;
            if (dm!=null)
            {
                dm.Saving-=ItemSaving;

                Dictionary.Remove(dm.Id); // Remove the reference with the temporary key
            }
        }

        /// <summary>Action performed when an item of the collection is saved.</summary>
        private void ItemSaved(object sender, PersistenceEventArgs e)
        {
            IDomainEntity dm=(IDomainEntity)sender;
            if (dm!=null)
            {
                dm.Saved-=ItemSaved;

                Dictionary.Add(dm.Id, (TDomainEntity)dm); // Add the final reference
            }
        }

        /// <summary>Resets the internal state of the collection.</summary>
        internal void Reset()
        {
            _AddedItems.Clear();
            _RemovedItems.Clear();
        }

        private List<TDomainEntity> _AddedItems=new List<TDomainEntity>();
        private List<TDomainEntity> _RemovedItems=new List<TDomainEntity>();
        /// <summary>Persistence lock used to avoid reentrance.</summary>
        private PersistenceLock _DeleteLock;
        /// <summary>Persistence lock used to avoid reentrance.</summary>
        private PersistenceLock _SaveLock;

        /// <summary>Occurs when the collection changes, either by adding or removing an item.</summary>
        public event CollectionChangeEventHandler CollectionChanged;
        /// <summary>Occurs when the instance is about to be deleted (<see cref="Delete(IPersistenceContext)" />).</summary>
        public event EventHandler<PersistenceEventArgs> Deleting;
        /// <summary>Occurs when the instance has been deleted (<see cref="Delete(IPersistenceContext)" />).</summary>
        public event EventHandler<PersistenceEventArgs> Deleted;
        /// <summary>Occurs when the instance is about to be saved (<see cref="Save(IPersistenceContext)" />).</summary>
        public event EventHandler<DomainEntityCollectionSavingEventArgs<TDomainEntity>> Saving
        {
            add
            {
                _Saving+=new EventHandler<PersistenceEventArgs>(
                    delegate(object sender, PersistenceEventArgs e) {
                        value(sender, (DomainEntityCollectionSavingEventArgs<TDomainEntity>)e);
                    }
                );
            }
            remove
            {
                _Saving-=new EventHandler<PersistenceEventArgs>(
                    delegate(object sender, PersistenceEventArgs e) {
                        value(sender, (DomainEntityCollectionSavingEventArgs<TDomainEntity>)e);
                    }
                );
            }
        }

        event EventHandler<PersistenceEventArgs> IPersistent.Saving
        {
            add
            {
                _Saving+=value;
            }
            remove
            {
                _Saving-=value;
            }
        }
        /// <summary>Occurs when the instance has been saved (<see cref="Save(IPersistenceContext)" />).</summary>
        public event EventHandler<PersistenceEventArgs> Saved;

        private event EventHandler<PersistenceEventArgs> _Saving;


    }
}
