// $Id$
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*! \file
 * \brief Generic interfaces of persistent objects.
 *
 * \author $Author$
 * \date $Date$
 * \version $Revision$
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;

namespace Salamanca.DataAccess.Data
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Implementation of a persistence context.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public sealed class PersistenceContext:
        IPersistenceContext
    {

        /// <summary>Constructor.</summary>
        public PersistenceContext(IDbConnection connection):
            this(connection, false)
        {
        }

        /// <summary>Constructor.</summary>
        public PersistenceContext(IDbConnection connection, bool forceAction)
        {
            _ForceAction=forceAction;
            _Transaction=new DataAccess.Transaction(connection);

            // We own the transaction
            _DisposeTransaction=true;
        }

        /// <summary>Constructor.</summary>
        public PersistenceContext(bool forceAction, DataAccess.ITransaction transaction)
        {
            Debug.Assert(transaction!=null);
            if (transaction==null)
                throw new ArgumentNullException("transaction");

            _ForceAction=forceAction;
            _Transaction=transaction;
        }

        /// <summary>Finalizer.</summary>
        ~PersistenceContext()
        {
            Dispose(false);
        }

        /// <summary>Frees the resources of this instance.</summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>Commits the operations executed in this <see cref="PersistenceContext" />.</summary>
        /// <remarks>This method does nothing more than <see cref="DataAccess.ITransaction.Commit" /> the current <see cref="Transaction" />.</remarks>
        public void Commit()
        {
            _Transaction.Commit();
        }

        /// <summary>Frees the resources of this instance.</summary>
        private void Dispose(bool disposing)
        {
            if (!_Disposed)
            {
                if (_DisposeTransaction)
                    // We own the transaction
                    Transaction.Dispose();

                _Disposed=true;
            }
        }

        /// <summary>Indicates whether the persistence action is forced.</summary>
        public bool ForceAction
        {
            get
            {
                return _ForceAction;
            }
        }

        /// <summary>The <see cref="DataAccess.ITransaction">transaction</see> in which the persistence action is executed.</summary>
        public DataAccess.ITransaction Transaction
        {
            get
            {
                return _Transaction;
            }
        }

        private bool _ForceAction;
        private DataAccess.ITransaction _Transaction;
        private bool _Disposed;
        private bool _DisposeTransaction;
    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Implementation of a persistence provider.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public sealed class PersistenceContextProvider:
        IPersistenceContextProvider
    {

        /// <summary>Initializes a new instance of the <see cref="PersistenceContextProvider" /> class.</summary>
        public PersistenceContextProvider(IDbConnection connection)
        {
            _Connection=connection;
        }

        /// <summary>Creates a persistence context.</summary>
        public IPersistenceContext CreatePersistenceContext()
        {
            return new PersistenceContext(_Connection);
        }

        /// <summary>Creates a persistence context.</summary>
        public IPersistenceContext CreatePersistenceContext(bool forceAction)
        {
            return new PersistenceContext(_Connection, forceAction);
        }

        private IDbConnection _Connection;
    }
}
