// $Id: DataMapper.cs 10 2008-02-29 15:58:02Z mcartoixa $
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*! \file
 * \brief Interfaces of Data Mappers.
 *
 * \author $Author: mcartoixa $
 * \date $Date: 2008-02-29 16:58:02 +0100 (ven., 29 févr. 2008) $
 * \version $Revision: 10 $
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;

namespace Salamanca.DataAccess
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Interface of a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DataMapper']/*" />.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public interface IDataMapper
    {

        /// <summary>Finds the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> associated to the specified key.</summary>
        IDomainEntity Find(IPrimaryKey key, Collections.DataMapperCollection dataMappers);

        /// <summary>Refreshes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be refreshed.</param>
        void Refresh(IDomainEntity domainModel);

        /// <summary>Deletes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        //        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be deleted.</param>
        void Delete(IDomainEntity domainModel, IPersistenceContext context);
        /// <summary>Saves the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        //        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be saved.</param>
        void Save(IDomainEntity domainModel, IPersistenceContext context);

        /// <summary>Gets the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> corresponding to the specified data.</summary>
        IDomainEntity GetDomainEntity(IDomainEntityDataTransferObject data, Collections.DataMapperCollection dataMappers);

        /// <summary>Gets the data associated to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        IDomainEntityDataTransferObject GetData(IDomainEntity domainModel, bool deep);
        /// <summary>Assigns the specified data to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        void SetData(IDomainEntity domainModel, IDomainEntityDataTransferObject data);
    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Base class for a <include file="../../doc/Global.doc.xml" path="doc/template[@name='DataMapper']/*" />.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public abstract class DataMapperBase<TDomainEntity, TPrimaryKey, TDataTransferObject>:
        IDataMapper
        where TDomainEntity:
            DomainEntity<TPrimaryKey, TDataTransferObject>
        where TPrimaryKey:
            IPrimaryKey,
            new()
        where TDataTransferObject:
            class, // SOAP serialization does not accept structs...
            IDataTransferObject,
            new()
    {

        /// <summary>Constructor.</summary>
        private DataMapperBase()
        {
        }

        /// <summary>Constructor.</summary>
        /// <param name="instanceCreator">Member used to create new instances for the type. Must not be <c>null</c>.</param>
        /// <param name="cacheManager">A cache manager. Must not be <c>null</c>.</param>
        /// <exception cref="ArgumentNullException">Raised if <paramref name="instanceCreator" /> is <c>null</c>.</exception>
        protected DataMapperBase(DomainEntityCreator<TDomainEntity> instanceCreator, ICacheManager cacheManager)
        {
            Debug.Assert(instanceCreator!=null);
            if (instanceCreator==null)
                throw new ArgumentNullException("instanceCreator", Resources.SR.NullDomainEntityCreator);
            Debug.Assert(cacheManager!=null);
            if (cacheManager==null)
                throw new ArgumentNullException("cacheManager");

            _InstanceCreator=instanceCreator;
            _CacheManager=cacheManager;
        }

        /// <summary>Gets the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> corresponding to the specified data.</summary>
        public TDomainEntity GetDomainEntity(IDomainEntityDataTransferObject data, Collections.DataMapperCollection dataMappers)
        {
            if (data==null)
                return default(TDomainEntity);

            TDomainEntity dm=CreateDomainEntity(data.Data);
            ((IDomainEntity)dm).DataMappers=dataMappers;
            SetData(dm, data);

            return GetInstance((TPrimaryKey)((IDomainEntity)dm).Id, dm);
        }

        /// <summary>Finds the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> associated to the specified key.</summary>
        public abstract TDomainEntity Find(TPrimaryKey key, Collections.DataMapperCollection dataMappers);

        /// <summary>Refreshes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        protected abstract void Refresh(TDomainEntity domainModel);

        /// <summary>Deletes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        protected abstract void Delete(TDomainEntity domainModel, IPersistenceContext context);

        /// <summary>Saves the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        protected abstract void Save(TDomainEntity domainModel, IPersistenceContext context);

        /// <summary>Gets the data associated to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        protected virtual IDomainEntityDataTransferObject GetData(TDomainEntity domainModel, bool deep)
        {
            return new DomainEntityDataTransferObject<TDataTransferObject>(
                (TDataTransferObject)((ITransferable)domainModel).GetData(),
                (deep ? ((IDomainEntity)domainModel).Childs : null),
                ((IDomainEntity)domainModel).IsLoaded,
                ((IDomainEntity)domainModel).IsModified,
                ((IDomainEntity)domainModel).IsNew
            );
        }

        /// <summary>Assigns the specified data to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        protected virtual void SetData(TDomainEntity domainModel, IDomainEntityDataTransferObject data)
        {
            ((ITransferable)domainModel).SetData(data.Data);
            domainModel.SetLoaded(data.IsLoaded);
            domainModel.SetModified(((DomainEntityDataTransferObject<TDataTransferObject>)data).IsModified);
            domainModel.SetNew(((DomainEntityDataTransferObject<TDataTransferObject>)data).IsNew);
        }

        /// <summary>Transfers the specified <param name="data" /> to the specified <param name="domainModel" />.</summary>
        internal protected virtual void TransferData(TDomainEntity domainModel, TDataTransferObject data)
        {
            SetData(domainModel, new DomainEntityDataTransferObject<TDataTransferObject>(data, null, true, false, false));
        }



        /// <summary>Creates a new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        protected TDomainEntity CreateDomainEntity(IDataTransferObject data)
        {
            return _InstanceCreator.Invoke(data);
        }

        /// <summary>Gets the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance with the specified key.</summary>
        /// <remarks>
        /// <para>If </para>
        /// <para>This method can be overriden to look for instances in a cache, or
        /// in an <a href="http://www.martinfowler.com/eaaCatalog/identityMap.html">Identity Map</a> for example.</para>
        /// <para>This method always returns <paramref name="newInstance" />.</para>
        /// </remarks>
        /// <param name="key">The key against which the command is created.</param>
        /// <param name="newInstance">An optional instance to return if no instance can be found.</param>
        /// <returns>The found <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance,
        /// <paramref name="newInstance" /> if no instance could be found (default).</returns>
        protected TDomainEntity GetInstance(TPrimaryKey key, TDomainEntity newInstance)
        {
            object ret=default(TDomainEntity);
            if ((!CacheManager.TryGetValue(key, out ret)) && (newInstance!=null))
            {
                IDomainEntity dm=newInstance as IDomainEntity;
                if ((dm==null) || (!dm.IsNew))
                    CacheManager.Add(key, newInstance);

                ret=newInstance;
            }

            return (TDomainEntity)ret;
        }

        /// <summary>Gets the <see cref="ICacheManager" /> associated to the
        /// <include file="../../doc/Global.doc.xml" path="doc/template[@name='DataMapper']/*" />.</summary>
        protected ICacheManager CacheManager
        {
            get
            {
                return _CacheManager;
            }
        }



        #region IDataMapper
        /// <summary>Finds the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> associated to the specified key.</summary>
        IDomainEntity IDataMapper.Find(IPrimaryKey key, Collections.DataMapperCollection dataMappers)
        {
            return Find((TPrimaryKey)key, dataMappers);
        }

        /// <summary>Refreshes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        /// <param name="domainModel">The <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance to be refreshed.</param>
        void IDataMapper.Refresh(IDomainEntity domainModel)
        {
            Refresh((TDomainEntity)domainModel);
        }

        /// <summary>Deletes the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        void IDataMapper.Delete(IDomainEntity domainModel, IPersistenceContext context)
        {
            Delete((TDomainEntity)domainModel, context);
        }

        /// <summary>Saves the data of the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</summary>
        void IDataMapper.Save(IDomainEntity domainModel, IPersistenceContext context)
        {
            Save((TDomainEntity)domainModel, context);
        }

        /// <summary>Gets the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> corresponding to the specified data.</summary>
        IDomainEntity IDataMapper.GetDomainEntity(IDomainEntityDataTransferObject data, Collections.DataMapperCollection dataMappers)
        {
            return GetDomainEntity(data, dataMappers);
        }

        /// <summary>Gets the data associated to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        IDomainEntityDataTransferObject IDataMapper.GetData(IDomainEntity domainModel, bool deep)
        {
            return GetData((TDomainEntity)domainModel, deep);
        }

        /// <summary>Assigns the specified data to the specified <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" />.</summary>
        void IDataMapper.SetData(IDomainEntity domainModel, IDomainEntityDataTransferObject data)
        {
            SetData((TDomainEntity)domainModel, data);
        }
        #endregion



        private DomainEntityCreator<TDomainEntity> _InstanceCreator;
        private ICacheManager _CacheManager;
    }

}
