﻿// $Id: DomainEntityCreator.cs 10 2008-02-29 15:58:02Z mcartoixa $
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*! \file
 * \brief Definition of \c Salamanca.DataAccess.DomainEntityCreator.
 *
 * \author $Author: mcartoixa $
 * \date $Date: 2008-02-29 16:58:02 +0100 (ven., 29 févr. 2008) $
 * \version $Revision: 10 $
 */

using System;
using System.Collections.Generic;
using System.ComponentModel.Design.Serialization;
using System.Diagnostics;
using System.Reflection;
using System.Text;

namespace Salamanca.DataAccess
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Delegate that is used to create a new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance,
    /// based on the specified data.</summary>
    /// <typeparam name="TDomainEntity">The expected <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance type.</typeparam>
    /// <param name="data">The data used to create a new instance.</param>
    /// <returns>A new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</returns>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public delegate TDomainEntity DomainEntityCreateHandler<TDomainEntity>(IDataTransferObject data)
        where TDomainEntity:
            IDomainEntity;



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Functor used to create new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instances.</summary>
    /// <typeparam name="TDomainEntity">The expected <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance type.</typeparam>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public sealed class DomainEntityCreator<TDomainEntity>
        where TDomainEntity:
            IDomainEntity
    {

        /// <summary>Constructor.</summary>
        private DomainEntityCreator()
        {
        }

        /// <summary>Constructor.</summary>
        /// <param name="handler">The delegate used to create a new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</param>
        /// <param name="list">An optional list of handlers. These must be <see cref="DomainEntityCreateHandler{TDomainEntity}" /> instances or <see cref="MemberInfo" /> instances.</param>
        public DomainEntityCreator(DomainEntityCreateHandler<TDomainEntity> handler, params object[] list)
        {
            Debug.Assert(handler!=null);
            if (handler==null)
                throw new ArgumentNullException("handler");

            AddHandlersOrMembers(handler, list);
        }

        /// <summary>Constructor.</summary>
        /// <param name="member">The member used to create a new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.
        /// During its invocation, the member will be passed a <see cref="IDataTransferObject" /> instance.</param>
        /// <param name="list">An optional list of handlers. These must be <see cref="DomainEntityCreateHandler{TDomainEntity}" /> instances or <see cref="MemberInfo" /> instances.</param>
        public DomainEntityCreator(MemberInfo member, params object[] list)
        {
            Debug.Assert(member!=null);
            if (member==null)
                throw new ArgumentNullException("member");

            AddHandlersOrMembers(GetHandler(member), list);
        }

        /// <summary>Invokes the functor with the specified data.</summary>
        /// <remarks>The functor will call all the registered handlers until one returns a non null <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</remarks>
        /// <param name="data">The data passed to the functor.</param>
        /// <returns>A new <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> instance.</returns>
        public TDomainEntity Invoke(IDataTransferObject data)
        {
            TDomainEntity ret=default(TDomainEntity);

            foreach (DomainEntityCreateHandler<TDomainEntity> handler in _HandlerList)
            {
                ret=handler(data);
                if (ret!=null)
                    return ret;
            }

            return Activator.CreateInstance<TDomainEntity>();
        }

        /// <summary>Adds a list of handlers.</summary>
        private void AddHandlersOrMembers(DomainEntityCreateHandler<TDomainEntity> first, object[] list)
        {
            if (list==null)
                list=new object[0];

            _HandlerList=new List<DomainEntityCreateHandler<TDomainEntity>>(list.Length+1);
            _HandlerList.Add(first);

            foreach (object item in list)
            {
                DomainEntityCreateHandler<TDomainEntity> h=item as DomainEntityCreateHandler<TDomainEntity>;
                if (h==null)
                {
                    MemberInfo mi=item as MemberInfo;
                    if (mi==null)
                    {
                        Debug.Assert(false);
                        throw new ArgumentException("list");
                    } else
                        h=GetHandler(mi);
                }

                _HandlerList.Add(h);
            }
        }

        /// <summary>Gets a <see cref="DomainEntityCreateHandler{TDomainEntity}" /> from the specified member.</summary>
        private static DomainEntityCreateHandler<TDomainEntity> GetHandler(MemberInfo member)
        {
            return new DomainEntityCreateHandler<TDomainEntity>(
                delegate(IDataTransferObject data) {
                    InstanceDescriptor descriptor=new InstanceDescriptor(member, new object[] { data }, true);
                    return (TDomainEntity)descriptor.Invoke();
                }
            );
        }

        private List<DomainEntityCreateHandler<TDomainEntity>> _HandlerList;

    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Static methods to handle <see cref="DomainEntityCreator{TDomainEntity}" /> instances.</summary>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public static class DomainEntityCreator
    {

        /// <summary>Creates a default functor (will use the <include file="../../doc/Global.doc.xml" path="doc/template[@name='DomainEntity']/*" /> default constructor).</summary>
        public static DomainEntityCreator<TDomainEntity> CreateDefault<TDomainEntity>()
            where TDomainEntity:
                IDomainEntity,
                new()
        {
            return new DomainEntityCreator<TDomainEntity>(
                new DomainEntityCreateHandler<TDomainEntity>(
                    delegate(IDataTransferObject data) {
                        return new TDomainEntity();
                    }
                )
            );
        }
    }

}
