// $Id: StringLengthRule.cs 18932 2009-03-27 15:56:17Z unknown $

/* ***************************************************************************
 *                                                                           *
 * Copyright  2007 NourY Solutions                                          *
 *                                                                           *
 * This file is part of DataRules.                                           *
 *                                                                           *
 * DataRules is free software; you can redistribute it and/or modify it      *
 * under the terms of the GNU Lesser General Public License as published     *
 * by the Free Software Foundation; either version 3 of the License, or      *
 * (at your option) any later version.                                       *
 *                                                                           *
 * DataRules is distributed in the hope that it will be useful, but          *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU          *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with this program.  If not, see                             *
 * <http://www.gnu.org/licenses/lgpl.html>.                                  *
 *                                                                           *
 *************************************************************************** */

/*! \file
 * \brief Definition of \c Salamanca.DataRules.RegexRule.
 *
 * \author $Author: unknown $
 * \date $Date: 2009-03-27 08:56:17 -0700 (周五, 27 三月 2009) $
 * \version $Revision: 18932 $
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Text;
using System.Text.RegularExpressions;

namespace Salamanca.DataRules
{



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Business rule based on the length of a string.</summary>
    /// <remarks>
    ///   <para>The length that is compared is the length of the <see cref="Object.ToString()">string representation</see> of the property value.</para>
    ///   <note>If the property value is <c>null</c>, the rule is considered validated.
    /// Add a <see cref="NotNullRule{T}" /> to the property if it cannot be <c>null</c>.</note>
    /// </remarks>
    /// <example>
    ///   <code>
    /// using Salamanca.DataRules;
    /// 
    /// public class City
    /// {
    ///     private string _ZipCode;
    /// 
    ///     public string ZipCode
    ///     {
    ///         get
    ///         {
    ///              return _ZipCode;
    ///         }
    ///     }
    /// 
    ///     protected List&lt;IRule&gt; CreateRules()
    ///     {
    ///         List&lt;IRule&gt; ret=new List&lt;IRule&gt;();
    /// 
    ///         ret.Add(
    ///             new StringLengthRule&lt;City&gt;(
    ///                 "ZipCode",
    ///                 "A Zip code must be exactly 5 characters long.",
    ///                 5,
    ///                 5
    ///             )
    ///         );
    /// 
    ///         ret.Add(
    ///             new NotNullRule&lt;City&gt;(
    ///                 "ZipCode"
    ///             )
    ///         );
    /// 
    ///         return ret;
    ///     }
    /// }
    ///   </code>
    /// </example>
    ///
    ///////////////////////////////////////////////////////////////////////////

    public class StringLengthRule<T>:
        GenericRule<T>
    {

        /// <summary>Constructor.</summary>
        /// <param name="propertyName">The name of the property. This cannot be an empty string (<c>""</c>).</param>
        /// <param name="maxLength">The maximum length of the string (inclusive).</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///   <para><paramref name="maxLength" /> must be a positive number.</para>
        /// </exception>
        /// <exception cref="ArgumentNullException">
        ///   <para><paramref name="propertyName" /> is <c>null</c>.</para>
        /// </exception>
        public StringLengthRule(string propertyName, int maxLength):
            this(propertyName, string.Format(CultureInfo.CurrentCulture, Resources.SR.StringLengthMaxLengthRuleDescription, propertyName, maxLength), maxLength)
        {
        }

        /// <summary>Constructor.</summary>
        /// <param name="propertyName">The name of the property. This cannot be an empty string (<c>""</c>).</param>
        /// <param name="description">The description of the rule.</param>
        /// <param name="maxLength">The maximum length of the string (inclusive).</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///   <para><paramref name="maxLength" /> must be a positive number.</para>
        /// </exception>
        /// <exception cref="ArgumentNullException">
        ///   <para><paramref name="propertyName" /> is <c>null</c>.</para>
        ///   <para><paramref name="description" /> is <c>null</c>.</para>
        /// </exception>
        public StringLengthRule(string propertyName, string description, int maxLength):
            this(propertyName, description, 0, maxLength)
        {
        }

        /// <summary>Constructor.</summary>
        /// <param name="propertyName">The name of the property. This cannot be an empty string (<c>""</c>).</param>
        /// <param name="maxLength">The maximum length of the string (inclusive).</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///   <para><paramref name="minLength" /> must be a positive number.</para>
        ///   <para><paramref name="maxLength" /> must be greater than <param name="minLength" />.</para>
        /// </exception>
        /// <exception cref="ArgumentNullException">
        ///   <para><paramref name="propertyName" /> is <c>null</c>.</para>
        /// </exception>
        public StringLengthRule(string propertyName, int minLength, int maxLength):
            this(propertyName, string.Format(CultureInfo.CurrentCulture, Resources.SR.StringLengthRuleDescription, propertyName, minLength, maxLength), minLength, maxLength)
        {
        }

        /// <summary>Constructor.</summary>
        /// <param name="propertyName">The name of the property. This cannot be an empty string (<c>""</c>).</param>
        /// <param name="description">The description of the rule.</param>
        /// <param name="minLength">The minimum length of the string (inclusive).</param>
        /// <param name="maxLength">The maximum length of the string (inclusive).</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///   <para><paramref name="minLength" /> must be a positive number.</para>
        ///   <para><paramref name="maxLength" /> must be greater than <paramref name="minLength" />.</para>
        /// </exception>
        /// <exception cref="ArgumentNullException">
        ///   <para><paramref name="propertyName" /> is <c>null</c>.</para>
        ///   <para><paramref name="description" /> is <c>null</c>.</para>
        /// </exception>
        public StringLengthRule(string propertyName, string description, int minLength, int maxLength):
            base(propertyName, description)
        {
            Debug.Assert(minLength>=0);
            if (minLength<0)
                throw new ArgumentOutOfRangeException(Resources.SR.MinLengthNegativeException, "minLength");
            Debug.Assert(maxLength>=minLength);
            if (maxLength<minLength)
                throw new ArgumentOutOfRangeException(Resources.SR.MaxLengthLessThanMinLengthException, "maxlength");

            _MinLength=minLength;
            _MaxLength=maxLength;
        }

        /// <summary>Validates the rule on the specified target.</summary>
        /// <remarks>
        ///   <para>The regular expression is matched against the <see cref="Object.ToString()">string representation</see> of the <see cref="GenericRule{T}.Property" /> value.</para>
        ///   <note>If the <see cref="GenericRule{T}.Property" /> value is <c>null</c>, this method returns <c>true</c>.
        /// Add a <see cref="NotNullRule{T}" /> to the <see cref="GenericRule{T}.Property" /> if it cannot be <c>null</c>.</note>
        /// </remarks>
        /// <param name="target">The instance against which the rule is validated.</param>
        /// <returns><c>true</c> if the instance is valid for the rule, <c>false</c> if not.</returns>
        protected override bool DoValidate(object target)
        {
            object v=Property.GetValue(target, null);
            if (v==null)
                return true;

            int l=v.ToString().Length;

            return (l>=_MinLength) && (l<=_MaxLength);
        }

        /// <summary>The minimum length of the string.</summary>
        private int _MinLength;
        /// <summary>The maximum length of the string.</summary>
        private int _MaxLength;
    }



    ///////////////////////////////////////////////////////////////////////////
    ///
    /// <summary>Business rule attribute based on the length of a string.</summary>
    /// <remarks>
    ///   <para>The length that is compared is the length of the <see cref="Object.ToString()">string representation</see> of the property value.</para>
    ///   <note>If the property value is <c>null</c>, the rule is considered validated.
    /// Add a <see cref="NotNullRule{T}" /> to the property if it cannot be <c>null</c>.</note>
    /// </remarks>
    /// <example>
    ///   <code>
    /// using Salamanca.DataRules;
    /// 
    /// public class City
    /// {
    ///     private string _ZipCode;
    /// 
    ///     [StringLengthRule("A Zip code must be exactly 5 characters long.", 5, 5)]
    ///     [NotNullRule]
    ///     public string ZipCode
    ///     {
    ///         get
    ///         {
    ///              return _ZipCode;
    ///         }
    ///     }
    /// 
    ///     protected List&lt;IRule&gt; CreateRules()
    ///     {
    ///         return Rules.CreateAttributedRule(GetType());
    ///     }
    /// }
    ///   </code>
    /// </example>
    ///
    ///////////////////////////////////////////////////////////////////////////

    [AttributeUsage(AttributeTargets.Property, Inherited=true, AllowMultiple=false)]
    public sealed class StringLengthRuleAttribute:
        RuleAttribute
    {

        /// <summary>Constructor.</summary>
        /// <param name="maxLength">The maximum length of the string (inclusive).</param>
        public StringLengthRuleAttribute(int maxLength):
            base(typeof(StringLengthRule<>))
        {
            _MaxLength=maxLength;
        }

        /// <summary>Gets the rule associated with the attribute.</summary>
        public override IRule GetRule(Type targetType, string propertyName)
        {
            if (string.IsNullOrEmpty(Description))
                return (IRule)typeof(StringLengthRule<>)
                    .MakeGenericType(targetType)
                    .GetConstructor(new Type[] { typeof(string), typeof(int), typeof(int) })
                    .Invoke(new object[] { propertyName, MinLength, MaxLength });
            else
                return (IRule)typeof(StringLengthRule<>)
                    .MakeGenericType(targetType)
                    .GetConstructor(new Type[] { typeof(string), typeof(string), typeof(int), typeof(int) })
                    .Invoke(new object[] { propertyName, Description, MinLength, MaxLength });
        }

        /// <summary>The maximum length of the string.</summary>
        public int MaxLength
        {
            get
            {
                return _MaxLength;
            }
        }

        /// <summary>The minimum length of the string.</summary>
        public int MinLength
        {
            get
            {
                return _MinLength;
            }
            set
            {
                _MinLength=value;
            }
        }

        /// <summary>The minimum length of the string.</summary>
        private int _MinLength;
        /// <summary>The maximum length of the string.</summary>
        private int _MaxLength;
    }

}
