// $Id: DomainEntity.cs 10 2008-02-29 15:58:02Z mcartoixa $
/*
 *  Copyright (C) 2007 NourY Solutions
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Reflection;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Salamanca.DataAccess;
using Salamanca.DataAccess.Collections;
using Salamanca.DataRules;
using Northwind;
using Northwind.DomainModel;

namespace Salamanca.NUnit.DataAccess.Northwind
{

    public abstract class DomainEntityBase
    {

        [TestMethod]
        public void CanDelete()
        {
            Customer c=Customer.Find("BONAP", _Mappers);
            Assert.IsTrue(((IDomainEntity)c).IsLoaded);
            Assert.IsFalse(((IDomainEntity)c).IsNew);
            Assert.IsFalse(((IDomainEntity)c).IsModified);

            c.Delete();

            Assert.IsFalse(((IDomainEntity)c).IsLoaded);
            Assert.IsTrue(((IDomainEntity)c).IsNew);
            Assert.IsTrue(((IDomainEntity)c).IsModified);

            Assert.IsNull(Customer.Find("BONAP", _Mappers));
        }

        [TestMethod]
        public void CanFindAllElements()
        {
            IList<Customer> d=Customer.FindAll(_Mappers);

            Assert.AreEqual(91, d.Count);

            Assert.IsTrue(
                (
                 (ICollection)typeof(DomainEntityKeyedCollection<Customer>)
                    .GetField("_AddedItems", BindingFlags.Instance | BindingFlags.NonPublic)
                    .GetValue(d)
                ).Count==0
            );
            Assert.IsTrue(
                (
                 (ICollection)typeof(DomainEntityKeyedCollection<Customer>)
                    .GetField("_RemovedItems", BindingFlags.Instance | BindingFlags.NonPublic)
                    .GetValue(d)
                ).Count==0
            );
        }

        [TestMethod]
        public void CanFindElementInList()
        {
            IList<Customer> d=Customer.FindAll(_Mappers);
            Customer c=Customer.Find("ALFKI", _Mappers);

            CollectionAssert.Contains((IList)d, c);
        }

        [TestMethod]
        public void CanFindByKey()
        {
            Customer c=Customer.Find("ALFKI", _Mappers);

            Assert.AreEqual("Alfreds Futterkiste", c.CompanyName);
            Assert.AreEqual("Maria Anders", c.ContactName);
            Assert.AreEqual("Sales Representative", c.ContactTitle);
            Assert.AreEqual("Obere Str. 57", c.Address);
            Assert.AreEqual("Berlin", c.City);
            Assert.IsNull(c.Region);
            Assert.AreEqual("12209", c.PostalCode);
            Assert.AreEqual("Germany", c.Country);
            Assert.AreEqual("030-0074321", c.Phone);
            Assert.AreEqual("030-0076545", c.Fax);
        }

        [TestMethod]
        public void CanModifyFieldAndDiscardChanges()
        {
            Customer c=Customer.Find("ANATR", _Mappers);

            Assert.AreEqual("Ana Trujillo Emparedados y helados", c.CompanyName);

            c.CompanyName="foo";
            Assert.AreEqual("foo", c.CompanyName);

            c.Refresh(true);
            Assert.AreEqual("Ana Trujillo Emparedados y helados", c.CompanyName);
        }

        [TestMethod]
        public void CanModifyFieldAndSaveChanges()
        {
            Customer c=Customer.Find("ANTON", _Mappers);

            Assert.AreEqual("Antonio Moreno", c.ContactName);

            c.ContactName="foo";
            Assert.IsTrue(((IDomainEntity)c).IsModified);

            c.Save();

            Assert.IsFalse(((IDomainEntity)c).IsModified);
            Assert.AreEqual("foo", c.ContactName);

            c.Refresh(true);
            Assert.AreEqual("foo", c.ContactName);
        }

        [TestMethod]
        public void CanModifyFieldAndRollbackTransaction()
        {
            Customer c1=Customer.Find("BERGS", _Mappers);
            Customer c2=Customer.Find("BLAUS", _Mappers);

            Assert.AreEqual("Christina Berglund", c1.ContactName);
            Assert.AreEqual("Hanna Moos", c2.ContactName);

            c1.ContactName="foo";
            c2.ContactName="foo";

            using (IPersistenceContext pc=Mappers.PersistenceContextProvider.CreatePersistenceContext())
            {
                c1.Save(pc);
                Assert.AreEqual("foo", c1.ContactName);

                c2.Save(pc);
                Assert.AreEqual("foo", c2.ContactName);

                // Do not call Commit : automatic rollback
            }

            Assert.AreEqual("foo", c1.ContactName);
            Assert.AreEqual("foo", c2.ContactName);

            c1.Refresh(true);
            Assert.AreEqual("Christina Berglund", c1.ContactName);

            c2.Refresh(true);
            Assert.AreEqual("Hanna Moos", c2.ContactName);
        }

        [TestMethod]
        public void CanModifyFieldAndCommitTransaction()
        {
            Customer c1=Customer.Find("BLONP", _Mappers);
            Customer c2=Customer.Find("BOLID", _Mappers);

            Assert.AreEqual("Frdrique Citeaux", c1.ContactName);
            Assert.AreEqual("Martn Sommer", c2.ContactName);

            c1.ContactName="foo";
            c2.ContactName="foo";

            using (IPersistenceContext pc=Mappers.PersistenceContextProvider.CreatePersistenceContext())
            {
                c1.Save(pc);
                Assert.AreEqual("foo", c1.ContactName);

                c2.Save(pc);
                Assert.AreEqual("foo", c2.ContactName);

                pc.Commit();
            }

            Assert.AreEqual("foo", c1.ContactName);
            Assert.AreEqual("foo", c2.ContactName);

            c1.Refresh(true);
            Assert.AreEqual("foo", c1.ContactName);

            c2.Refresh(true);
            Assert.AreEqual("foo", c2.ContactName);
        }

        [TestMethod]
        public void CanCreateInstanceAndSaveChanges()
        {
            Customer c=new Customer("UTEST", _Mappers);
            c.CompanyName="Test";
            c.ContactName="Test";
            c.ContactTitle="Test";
            c.Address="Test";
            c.City="Test";
            c.PostalCode="Test";
            c.Country="Test";
            c.Phone="Test";
            c.Fax="Test";
            c.Region="Test";

            Assert.IsFalse(((IDomainEntity)c).IsLoaded);
            Assert.IsTrue(((IDomainEntity)c).IsNew);
            Assert.IsTrue(((IDomainEntity)c).IsModified);

            c.Save();

            Assert.IsTrue(((IDomainEntity)c).IsLoaded);
            Assert.IsFalse(((IDomainEntity)c).IsNew);
            Assert.IsFalse(((IDomainEntity)c).IsModified);
        }

        [TestMethod]
        public void CanCreateInstanceWithGeneratedKeyAndSaveChanges()
        {
            Employee e=new Employee(_Mappers);
            e.FirstName="Test";
            e.LastName="Test";
            e.HireDate=DateTime.Now;
            e.BirthDate=new DateTime(1970, 1, 1);

            Assert.IsFalse(((IDomainEntity)e).IsLoaded);
            Assert.IsTrue(((IDomainEntity)e).IsNew);
            Assert.IsTrue(((IDomainEntity)e).IsModified);

            e.Save();

            Assert.IsTrue(((IDomainEntity)e).IsLoaded);
            Assert.IsFalse(((IDomainEntity)e).IsNew);
            Assert.IsFalse(((IDomainEntity)e).IsModified);
            Assert.IsInstanceOfType(((IDomainEntity)e).Id.Value, typeof(Int32));
        }

        [TestMethod, ExpectedException(typeof(BrokenRulesException))]
        public void CannotSaveInvalidInstance()
        {
            Customer c=new Customer("TEST", _Mappers); // ID must be 5 characters long

            c.Save();
        }

        [TestMethod]
        public void CanFindChildren()
        {
            Customer c=Customer.Find("AROUT", _Mappers);

            Assert.AreEqual(13, c.Orders.Count);
        }

        [TestMethod]
        public void CanFindParent()
        {
            Order o=Order.Find(10278, _Mappers);

            Assert.AreEqual("BERGS", ((IDomainEntity)o.Customer).Id.Value);
        }

        [TestMethod]
        public void CanFindAllBase()
        {
            IList<Employee> employees=Employee.FindAll(_Mappers);

            Assert.AreEqual(9, employees.Count);

            foreach (Employee e in employees)
                Assert.IsInstanceOfType(e, typeof(Employee));
        }

        [TestMethod]
        public void CanBackup()
        {
            Customer c=Customer.Find("BONAP", _Mappers);
            IBackupable b=(IBackupable)c;

            Assert.AreEqual("Marseille", c.City);

            object backup=b.CreateBackup();
            c.City="Paris";
            Assert.AreEqual("Paris", c.City);

            b.RestoreBackup(backup);
            Assert.AreEqual("Marseille", c.City);
        }

        [TestMethod]
        public void CanCompareElements()
        {
            Customer c1=Customer.Find("BOTTM", _Mappers);
            Customer c2=Customer.Find("BSBEV", _Mappers);
            Customer c3=Customer.Find("BOTTM", _Mappers);

            Assert.IsNotNull(c1);
            Assert.IsNotNull(c2);
            Assert.IsNotNull(c3);

            Assert.AreNotEqual(c1, c2);
            Assert.AreEqual(c1, c3);

            Assert.IsTrue(c1!=c2);
            Assert.IsTrue(c1==c3);

            Assert.AreNotSame(c1, c2);
        }

        [TestMethod]
        public void CanAddNewItemToKeyedCollection()
        {
            Collection<Customer> d=new Collection<Customer>(Customer.FindAll(_Mappers));

            Assert.AreEqual(91, d.Count);

            Customer c=new Customer("UTEST", _Mappers);
            c.CompanyName="Test";
            c.ContactName="Test";
            c.ContactTitle="Test";
            c.Address="Test";
            c.City="Test";
            c.PostalCode="Test";
            c.Country="Test";
            c.Phone="Test";
            c.Fax="Test";
            c.Region="Test";

            d.Add(c);
            Assert.AreEqual(92, d.Count);
            CollectionAssert.Contains(d, c);

            c.Save();

            Assert.AreEqual(92, d.Count);
            CollectionAssert.Contains(d, c);
        }

        protected DataMapperCollection Mappers
        {
            get
            {
                return _Mappers;
            }
            set
            {
                _Mappers=value;
            }
        }

        private DataMapperCollection _Mappers;

    }
}
