﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using DslModeling=global::Microsoft.VisualStudio.Modeling;
using DslDesign=global::Microsoft.VisualStudio.Modeling.Design;
using DslValidation=global::Microsoft.VisualStudio.Modeling.Validation;

namespace Salamanca.Sdml
{
    public interface IDataTableHolder
    {
        DataTable DataTable { get; }
    }

    [DslValidation.ValidationState(DslValidation.ValidationState.Enabled)]
    partial class DomainModel
    {

        internal void LoadDataSet()
        {
            if (_DataSet==null)
            {
                DataSet ds=null;
                if (string.IsNullOrEmpty(Name))
                    ds=new DataSet();
                else
                    ds=new DataSet(Name);

                DataTable dt;
                DataColumn dc;

                // Sort the elements in reference order in order to generate tables in the correct order
                List<DslModeling.ModelElement> elements=new List<DslModeling.ModelElement>(Partition.ElementDirectory.AllElements)
                    .FindAll(
                        delegate(DslModeling.ModelElement me) {
                            // Get domain entities...
                            if ((me is DomainEntity) || (me is DomainEnum))
                                return true;

                            // ... and n-n associations.
                            Association association=me as Association;
                            if (association!=null)
                                return (!Association.IsSingleMultiplicity(association.SourceMultiplicity) && !Association.IsSingleMultiplicity(association.TargetMultiplicity));

                            return false;
                        }
                    );
                elements.Sort(
                    delegate(DslModeling.ModelElement me1, DslModeling.ModelElement me2) {
                        if (me1==me2)
                            return 0;

                        // nulls come first
                        if (me1==null)
                            return -1;
                        if (me2==null)
                            return 1;

                        // Domain Enums come first
                        DomainEnum dem1=me1 as DomainEnum;
                        DomainEnum dem2=me2 as DomainEnum;
                        if ((dem1!=null) || (dem2!=null))
                        {
                            if ((dem1!=null) && (dem2==null))
                                return -1;
                            if ((dem1==null) && (dem2!=null))
                                return 1;

                            return dem1.CompareTo(dem2);
                        }

                        // Domain Entities come second
                        DomainEntity dey1=me1 as DomainEntity;
                        DomainEntity dey2=me2 as DomainEntity;
                        if ((dey1!=null) || (dey2!=null))
                        {
                            if ((dey1!=null)&&(dey2==null))
                                return -1;
                            if ((dey1==null)&&(dey2!=null))
                                return 1;

                            return dey1.CompareTo(dey2);
                        }

                        return 0;
                    }
                );

                // Create a table for every element
                foreach (DslModeling.ModelElement element in elements)
                {

                    // Enumerations
                    DomainEnum enumeration=element as DomainEnum;
                    if (enumeration!=null)
                    {
                        dt=ds.Tables.Add(enumeration.Name);
                        enumeration.DataTable=dt;

                        SdmlType ct=TypesProvider.GetType(enumeration);
                        dc=dt.Columns.Add("Code", ct.DataType);
                        dc.MaxLength=((IAttribute)enumeration).MaxLength;
                        dc.ExtendedProperties.Add(typeof(SdmlType), ct);
                        dt.Constraints.Add(
                            string.Concat("PK_", dt.TableName),
                            dc,
                            true
                        );

                        ct=TypesProvider.GetType(new Attribute("string", 255));
                        dc=dt.Columns.Add("Description", ct.DataType);
                        dc.ExtendedProperties.Add(typeof(SdmlType), ct);
                    }

                    // Entities
                    DomainEntity entity=element as DomainEntity;
                    if (entity!=null)
                    {
                        dt=ds.Tables.Add(entity.TableName);
                        dt.ExtendedProperties.Add(typeof(DslModeling.ModelElement), entity);
                        entity.DataTable=dt;

                        // Primary keys
                        if (entity.Parent!=null)
                        {
                            List<DataColumn> keys=new List<DataColumn>();
                            keys.AddRange(
                                CreateColumnsFromReferenceTable(
                                    dt,
                                    ds.Tables[entity.Parent.TableName],
                                    Rule.Cascade,
                                    Rule.Cascade,
                                    null
                                )
                            );

                            dt.Constraints.Add(
                                string.Concat("PK_", dt.TableName),
                                keys.ToArray(),
                                true
                            );
                            dt.ParentRelations[dt.ParentRelations.Count-1].Nested=true;
                        } else if (entity.MultipleAssociation!=null)
                        {
                            List<DataColumn> keys=new List<DataColumn>();
                            foreach (DomainEntity associated in entity.MultipleAssociation.Types)
                            {
                                keys.AddRange(
                                    CreateColumnsFromReferenceTable(
                                        dt,
                                        ds.Tables[associated.TableName],
                                        Rule.None,
                                        Rule.None,
                                        associated.Name
                                    )
                                );
                            }

                            dt.Constraints.Add(
                                string.Concat("PK_", dt.TableName),
                                keys.ToArray(),
                                true
                            );
                        } else
                        {
                            dc=dt.Columns.Add("Id", entity.IdType.DataType);
                            dc.ExtendedProperties.Add(typeof(SdmlType), entity.IdType);

                            dt.Constraints.Add(
                                string.Concat("PK_", dt.TableName),
                                dc,
                                true
                            );
                        }

                        // Association foreign keys
                        List<Association> aList=new List<Association>(
                            DslModeling::DomainRoleInfo.GetElementLinks<Association>(
                                entity,
                                Association.TargetDomainRoleId
                            )
                        ).FindAll(
                            delegate(Association a) {
                                return Association.IsSingleMultiplicity(a.SourceMultiplicity);
                            }
                        );

                        foreach (Association a in aList)
                        {
                            DomainEntity referenced=a.Source;
                            string roleName=a.GetSourceRoleName();
                            Multiplicity multiplicity=a.SourceMultiplicity;

                            Rule rule=Rule.None;
                            switch (a.Form)
                            {
                            case AssociationForm.Composition:
                                rule=Rule.Cascade;
                                break;
                            case AssociationForm.Aggregation:
                                if (!Association.IsMandatoryMultiplicity(multiplicity))
                                    rule=Rule.SetNull;
                                break;
                            }

                            foreach (DataColumn column in CreateColumnsFromReferenceTable(
                                    dt,
                                    ds.Tables[a.Source.TableName],
                                    rule,
                                    rule,
                                    roleName
                                )
                            )
                            {
                                if (Association.IsMandatoryMultiplicity(multiplicity))
                                    column.AllowDBNull=false;
                            }
                            a.DataRelation=dt.ParentRelations[dt.ParentRelations.Count-1];
                        }

                        // Attributes
                        foreach (EntityAttribute attribute in entity.Attributes)
                        {
                            dc=null;

                            // If attribute type is an enumeration
                            if (Store.ElementDirectory.FindElements<DomainEnum>().Any<DomainEnum>(e => e.Name==attribute.Type))
                            {
                                foreach (DataColumn column in CreateColumnsFromReferenceTable(
                                        dt,
                                        ds.Tables[attribute.Type],
                                        attribute.AllowNull ? Rule.SetNull : Rule.None,
                                        attribute.AllowNull ? Rule.SetNull : Rule.None,
                                        attribute.Name
                                    )
                                )
                                {
                                    dc=column;
                                }
                            } else
                            {
                                SdmlType st=TypesProvider.GetType(attribute);

                                dc=dt.Columns.Add(attribute.Name, st.DataType);
                                dc.MaxLength=attribute.MaxLength;
                                dc.ExtendedProperties.Add(typeof(SdmlType), st);
                            }

                            dc.AllowDBNull=attribute.AllowNull;
                            attribute.DataColumn=dc;

                            if (attribute.Unique)
                                dt.Constraints.Add(
                                    string.Concat("UN_", dt.TableName, "_", dc.ColumnName),
                                    dc,
                                    true
                                );
                        }
                    }

                    // Create an association table for every n-n relation
                    Association association=element as Association;
                    if (association!=null)
                    {
                        if (Association.IsSingleMultiplicity(association.SourceMultiplicity) || Association.IsSingleMultiplicity(association.TargetMultiplicity))
                            continue;

                        dt=ds.Tables.Add(string.Concat(association.Source.TableName, association.Target.TableName));
                        dt.ExtendedProperties.Add(typeof(DslModeling.ModelElement), association);
                        association.DataTable=dt;

                        DomainEntity keySourceEntity=association.Source;
                        //while (keySourceEntity.Parent!=null)
                        //    keySourceEntity=keySourceEntity.Parent;
                        DomainEntity keyTargetEntity=association.Target;
                        //while (keyTargetEntity.Parent!=null)
                        //    keyTargetEntity=keyTargetEntity.Parent;

                        List<DataColumn> keys=new List<DataColumn>();
                        foreach (DomainEntity associated in new DomainEntity[] { keySourceEntity, keyTargetEntity })
                        {
                            keys.AddRange(
                                CreateColumnsFromReferenceTable(
                                    dt,
                                    ds.Tables[associated.TableName],
                                    Rule.None,
                                    Rule.None,
                                    associated.Name
                                )
                            );
                        }

                        dt.Constraints.Add(
                            string.Concat("PK_", dt.TableName),
                            keys.ToArray(),
                            true
                        );
                    }
                }

                _DataSet=ds;
                if (_ElementEventsEndedEventHandler==null)
                    _ElementEventsEndedEventHandler=_InitDataSet;
                Store.EventManagerDirectory.ElementEventsEnded.Add(_ElementEventsEndedEventHandler);
            }
        }

        private IEnumerable<DataColumn> CreateColumnsFromReferenceTable(DataTable table, DataTable referenceTable, Rule deleteRule, Rule updateRule, string referenceBaseName)
        {
            List<DataColumn> columns=new List<DataColumn>();
            foreach (DataColumn column in referenceTable.PrimaryKey)
            {
                DataColumn dc=table.Columns.Add(
                    string.IsNullOrEmpty(referenceBaseName) ? column.ColumnName : string.Concat(referenceBaseName, "_", column.ColumnName),
                    column.DataType
                );
                dc.ExtendedProperties.Add(typeof(SdmlType), column.ExtendedProperties[typeof(SdmlType)]);

                columns.Add(dc);
            }

            ForeignKeyConstraint fkc=new ForeignKeyConstraint(
                string.Concat("FK_", table.TableName, "_", referenceTable.TableName),
                referenceTable.PrimaryKey,
                columns.ToArray()
            );
            fkc.DeleteRule=deleteRule;
            fkc.UpdateRule=updateRule;

            table.Constraints.Add(fkc);

            DataRelation rel=new DataRelation(
                string.Concat(table.TableName, "_", referenceTable.TableName),
                referenceTable.PrimaryKey,
                columns.ToArray()
            );
            table.DataSet.Relations.Add(rel);

            return columns;
        }

        internal void LoadGroupedEntities()
        {
            if (_GroupedEntities==null)
            {
                List<IList<DomainEntity>> ret=GetGroupedEntities(
                    Partition.ElementDirectory.FindElements<DomainEntity>(),
                    new List<DomainEntity>()
                );

                foreach (IList<DomainEntity> group in ret)
                    ((List<DomainEntity>)group).Sort(
                        delegate(DomainEntity e1, DomainEntity e2) {
                            return e1.Name.CompareTo(e2.Name);
                        }
                    );

                _GroupedEntities=ret;
            }
        }

        /// <summary>Gets a list of lists of <see cref="DomainEntity">entities</see>, ordered by dependencies.</summary>
        /// <param name="entities">The entities to group by dependencies.</param>
        /// <param name="excluded">The collection of entities that will not be considered when checking the dependencies.</param>
        /// <returns>The list of lists of <see cref="DomainEntity">entities</see>, ordered by dependencies.</returns>
        /// <exception cref="InvalidOperationException">Thrown when at least one association loop has been detected.</exception>
        /// <remarks>
        /// <para></para>
        /// <para>The algorithm has been inspired by <a href="http://www.codeproject.com/KB/database/SortDBTablesByReference.aspx" target="_blank">this article on CodeProject</a>.</para>
        /// </remarks>
        private List<IList<DomainEntity>> GetGroupedEntities(IEnumerable<DomainEntity> entities, ICollection<DomainEntity> excluded)
        {
            List<DomainEntity> referencedOnly=new List<DomainEntity>();
            List<DomainEntity> referencesOnly=new List<DomainEntity>();
            List<DomainEntity> other=new List<DomainEntity>();

            foreach (DomainEntity entity in entities)
            {
                if (!entity.ReferencesEntities(excluded))
                {
                    referencedOnly.Add(entity);
                    entity.EntitiesGroup=referencedOnly;
                    continue;
                }

                if (!entity.IsReferenced(excluded))
                {
                    referencesOnly.Add(entity);
                    entity.EntitiesGroup=referencesOnly;
                    continue;
                }

                other.Add(entity);
            }

            List<DomainEntity> toExclude=new List<DomainEntity>(excluded);
            toExclude.AddRange(referencedOnly);
            toExclude.AddRange(referencesOnly);
            if ((other.Count>0) && (toExclude.Count==excluded.Count))
            {
                // Loop detected
                InvalidOperationException ex=new InvalidOperationException(
                    string.Format(
                        CultureInfo.CurrentCulture,
                        Resources.SR.AssociationLoopDetected,
                        other.Count
                    )
                );
                ex.Data.Add(typeof(DomainEntity), other.ConvertAll<DomainElement>(e => (DomainElement)e).ToArray());
                throw ex;
            }

            List<IList<DomainEntity>> ret=new List<IList<DomainEntity>>();
            if (referencedOnly.Count>0)
                ret.Add(referencedOnly);
            if (other.Count>0)
                ret.AddRange(GetGroupedEntities(other, toExclude));
            if (referencesOnly.Count>0)
                ret.Add(referencesOnly);

            return ret;
        }

        /// <summary>Checks that entity names are not duplicated in the model.</summary>
        [DslValidation.ValidationMethod(DslValidation.ValidationCategories.Menu | DslValidation.ValidationCategories.Open | DslValidation.ValidationCategories.Save)]
        private void ValidateName(DslValidation.ValidationContext context)
        {
            Dictionary<string, IList<DslModeling.ModelElement>> entities=new Dictionary<string, IList<Microsoft.VisualStudio.Modeling.ModelElement>>(StringComparer.InvariantCultureIgnoreCase);

            foreach (DomainElement element in Elements)
            {
                DomainEntity entity=element as DomainEntity;
                if (entity==null)
                    continue;

                if (!entities.ContainsKey(entity.Name))
                    entities.Add(entity.Name, new List<DslModeling.ModelElement>(new DslModeling.ModelElement[] { entity }));
                else
                    entities[entity.Name].Add(entity);
            }

            foreach (IList<DslModeling.ModelElement> duplicates in entities.Values.Where(list => list.Count>1))
                context.LogError(
                    string.Format(
                        CultureInfo.CurrentCulture,
                        Resources.SR.EntityNameCannotBeDuplicated,
                        ((DomainEntity)duplicates[0]).Name
                    ),
                    "SDML04",
                    duplicates.ToArray()
                );
        }

        /// <summary>Checks that entities associations are not looped.</summary>
        [DslValidation.ValidationMethod(DslValidation.ValidationCategories.Menu | DslValidation.ValidationCategories.Open | DslValidation.ValidationCategories.Save)]
        private void ValidateRelationLoops(DslValidation.ValidationContext context)
        {
            try
            {
                LoadGroupedEntities();
                _GroupedEntities=null;
            } catch (InvalidOperationException ioex)
            {
                if (!ioex.Data.Contains(typeof(DomainEntity)))
                    throw;

                context.LogError(
                    ioex.Message,
                    "SDML07",
                    (DomainElement[])ioex.Data[typeof(DomainEntity)]
                );
            }
        }

        private void _InitDataSet(object sender, EventArgs e)
        {
            Store.EventManagerDirectory.ElementEventsEnded.Remove(_ElementEventsEndedEventHandler);
            _DataSet=null;
            _GroupedEntities=null;
        }

        /// <summary>In-memory database structure implied by the current <see cref="DomainModel" />.</summary>
        public DataSet DataSet
        {
            get
            {
                LoadDataSet();
                return _DataSet;
            }
        }

        internal IList<IList<DomainEntity>> GroupedEntities
        {
            get
            {
                LoadGroupedEntities();
                return _GroupedEntities;
            }
        }

        private EventHandler<DslModeling.ElementEventsEndedEventArgs> _ElementEventsEndedEventHandler;

        private DataSet _DataSet;
        private List<IList<DomainEntity>> _GroupedEntities;
    }
}
