﻿using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using Microsoft.CSharp;
using DslModeling=global::Microsoft.VisualStudio.Modeling;

namespace Salamanca.Sdml
{

    public static class TypesProvider
    {

        /// <summary>Gets the <see cref="SdmlType" /> associated to the specified <paramref name="attribute" />.</summary>
        public static SdmlType GetType(IAttribute attribute)
        {
            EntityAttribute ea=attribute as EntityAttribute;
            if (ea!=null)
            {
                DomainEnum enumeration=ea.Partition.ElementDirectory.FindElements<DomainEnum>().FirstOrDefault<DomainEnum>(e => e.Name==attribute.Type);
                if (enumeration!=null)
                    return GetType(enumeration);
            }

            switch (attribute.Type)
            {
            case "bool":
                return new BooleanType(attribute);
            case "byte":
                return new ByteType(attribute);
            case "DateTime":
                return new DateTimeType(attribute);
            case "float":
                return new SingleType(attribute);
            case "double":
                return new DoubleType(attribute);
            case "int":
                return new Int32Type(attribute);
            case "long":
                return new Int64Type(attribute);
            case "short":
                return new Int16Type(attribute);
            case "Guid":
                return new GuidType(attribute);
            default:
                return new StringType(attribute);
            }
        }

        // cf. http://blog.visualt4.com/2009/03/transforming-type-names.html
        internal static string ToString(Type type)
        {
            using (StringWriter writer=new StringWriter())
            {
                new CSharpCodeProvider().GenerateCodeFromExpression(new CodeTypeReferenceExpression(type), writer, new CodeGeneratorOptions());
                return writer.ToString();
            }
        }
    }

    public abstract class SdmlType
    {
        public SdmlType(IAttribute attribute)
        {
            Debug.Assert(attribute!=null);
            if (attribute==null)
                throw new ArgumentNullException("attribute");

            _Attribute=attribute;
        }

        protected SdmlType()
        {
        }

        public virtual Type CSharpType
        {
            get
            {
                return DataType;
            }
        }

        public abstract Type DataType
        {
            get;
        }

        public abstract string SqlType
        {
            get;
        }

        public virtual string SqlDefault
        {
            get
            {
                return string.Empty;
            }
        }

        public abstract DbType DbType
        {
            get;
        }

        protected IAttribute Attribute
        {
            get
            {
                return _Attribute;
            }
        }

        private IAttribute _Attribute;
    }

    internal class DateTimeType:
        SdmlType
    {
        public DateTimeType(IAttribute attribute) :
            base(attribute)
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(DateTime);
            }
        }

        public override string SqlType
        {
            get
            {
                return "datetime";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.DateTime;
            }
        }
    }

    internal class BooleanType:
        SdmlType
    {
        public BooleanType(IAttribute attribute) :
            base(attribute)
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(bool);
            }
        }

        public override string SqlType
        {
            get
            {
                return "bit";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Boolean;
            }
        }
    }

    internal class SingleType:
        SdmlType
    {
        public SingleType(IAttribute attribute):
            base(attribute)
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(float);
            }
        }

        public override string SqlType
        {
            get
            {
                return "real";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Single;
            }
        }
    }

    internal class DoubleType:
        SdmlType
    {
        public DoubleType(IAttribute attribute) :
            base(attribute)
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(double);
            }
        }

        public override string SqlType
        {
            get
            {
                return "float";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Double;
            }
        }
    }

    internal class StringType:
        SdmlType
    {
        public StringType(IAttribute attribute) :
            base(attribute)
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(string);
            }
        }

        public override string SqlType
        {
            get
            {
                return string.Format(
                    CultureInfo.InvariantCulture,
                    "varchar({0})",
                    Attribute.MaxLength>0?(object)Attribute.MaxLength:"MAX"
                );
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.String;
            }
        }
    }

    internal class GuidType:
        SdmlType
    {
        public GuidType(IAttribute attribute):
            base(attribute)
        {
        }

        public GuidType()
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(Guid);
            }
        }

        public override string SqlType
        {
            get
            {
                return "uniqueidentifier";
            }
        }

        public override string SqlDefault
        {
            get
            {
                return "NEWSEQUENTIALID()";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Guid;
            }
        }
    }

    internal class ByteType:
        SdmlType
    {
        public ByteType(IAttribute attribute):
            base(attribute)
        {
        }

        public ByteType()
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(byte);
            }
        }

        public override string SqlType
        {
            get
            {
                return "tinyint";
            }
        }

        public override string SqlDefault
        {
            get
            {
                return "IDENTITY(1, 1)";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Byte;
            }
        }
    }

    internal class Int16Type:
        SdmlType
    {
        public Int16Type(IAttribute attribute) :
            base(attribute)
        {
        }

        public Int16Type()
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(short);
            }
        }

        public override string SqlType
        {
            get
            {
                return "smallint";
            }
        }

        public override string SqlDefault
        {
            get
            {
                return "IDENTITY(1, 1)";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Int16;
            }
        }
    }

    internal class Int32Type:
        SdmlType
    {
        public Int32Type(IAttribute attribute) :
            base(attribute)
        {
        }

        public Int32Type()
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(int);
            }
        }

        public override string SqlType
        {
            get
            {
                return "int";
            }
        }

        public override string SqlDefault
        {
            get
            {
                return "IDENTITY(1, 1)";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Int32;
            }
        }
    }

    internal class Int64Type:
        SdmlType
    {
        public Int64Type(IAttribute attribute) :
            base(attribute)
        {
        }

        public Int64Type()
        {
        }

        public override Type DataType
        {
            get
            {
                return typeof(long);
            }
        }

        public override string SqlType
        {
            get
            {
                return "bigint";
            }
        }

        public override string SqlDefault
        {
            get
            {
                return "IDENTITY(1, 1)";
            }
        }

        public override DbType DbType
        {
            get
            {
                return DbType.Int64;
            }
        }
    }
}
